var Sprig = function(x, y){
    this.x = x;
    this.y = y;
    this.moveX = 0;
    this.moveY = 0;
    this.visible = true;
};
var Sprite = function(imgSrc, x, y){
    Sprig.call(this, x, y);
    this.img = new Image();
    this.img.src = imgSrc;
    this.continuous = false;
};
Sprite.prototype.animate = function() {
    if(this.visible){
        ctx.drawImage(this.img, this.x, this.y);
    }
    if(game.state === game.states.running || game.state === game.states.over){
        this.x += this.moveX;
        this.y += this.moveY;
        if(this.continuous && this.x < (canvas.width - this.img.width)){
            this.x = 0;
        }
    }
};
Sprite.prototype.getRect = function(){
    return {x: this.x, y: this.y, width: this.img.width, height: this.img.height};
};
var Anim = function(imgSrc, frames, frameWidth, repeat){
    this.img = new Image();
    this.img.src = imgSrc;
    this.frames = frames;
    this.frameWidth = frameWidth;
    this.frame = -1;
    this.default = 0;
    this.repeat = repeat;
    this.pingPong = false;
    this.pp = 1;
    this.reset = function(){this.frame = -1;};
};
Anim.prototype.animate = function(x, y) {
    if(game.state == game.states.running){
        if(this.repeat){
            if (this.pingPong){
                this.frame += this.pp;
                if(this.frame >= this.frames || this.frame < 0){
                    this.pp *= -1;
                    this.frame += this.pp;
                }
            } else {
                this.frame++;
                this.frame %= this.frames;
            }
        } else {
            if(this.frame < this.frames -1){
                this.frame++;
            }
        }
    } else {
        this.frame = this.default; // stop animation
    }
    let clipx = this.frame * this.frameWidth;
    ctx.drawImage(this.img, clipx, 0, this.frameWidth, this.img.height, x, y, this.frameWidth, this.img.height);
};
Anim.prototype.getRect = function(x, y){
    return {x: x, y: y, width: this.frameWidth, height: this.img.height};
};
var Actor = function(x, y){
    Sprig.call(this, x, y);
    this.anims = [];
    this.current = 0;
    this.height = 0;
    this.resetCurrent = function(newVal){
        this.current = newVal;
        this.anims[this.current].reset();
    };
};
Actor.prototype.manageActions = function() {};
Actor.prototype.animate = function() {
    if(game.state == game.states.running){
        this.x += this.moveX;
        this.y += this.moveY;
    }
    if(this.visible){
        this.anims[this.current].animate(this.x, this.y);
    }
    this.manageActions();
};
Actor.prototype.getRect = function(){
    return this.anims[this.current].getRect(this.x, this.y);
};
Actor.prototype.collisionDetect = function(){
    let mRect = this.getRect();
    let collisions = [];
    for (let type in objects){
        if(type === "platforms"){continue;}
        let list = objects[type];
        for(let s = 0; s < list.length; s++){
            let sRect = list[s].getRect();
            if(bump(mRect, sRect)){
                collisions.push({type: type, index: s});
            }
        }
    }
    this.platformDetect(mRect, collisions);
    if(collisions.length > 0){
        this.collision(collisions);
    }
};
Actor.prototype.platformDetect = function(mRect, collisions){
    for(let p = 0; p < objects.platforms.length; p++){
        let sRect = objects.platforms[p].getRect();
        if (Math.abs(sRect.y - (mRect.y + mRect.height)) <= 16){
            let ft = mRect.x + mRect.width * 0.33;
            let lt = mRect.x + mRect.width * 0.66;
            if((ft >= sRect.x && ft <= sRect.x + sRect.width)||(lt >= sRect.x && lt <= sRect.x + sRect.width)){
                    collisions.push({type: "platform", index: p});
            }
        }
    }
};
Actor.prototype.collision = function(){};
var Coin = function(x,y){
    Actor.call(this, x, y);
    let a = new Anim("images/Coins5x22.png", 5, 22, true);
    a.frame = Math.floor(Math.random() * 5); // start at random frame
    this.anims.push(a);
    a = new Anim("images/stars10x33.png", 10, 33, false);
    a.default = 9;
    this.anims.push(a);
};
Coin.prototype = Object.create(Actor.prototype);
Coin.prototype.collision = function(){
    if(this.current !== 1){
        game.addCash(1);
        this.resetCurrent(1);
        sounds.coin.play();
    }
};
Coin.prototype.manageActions = function(){
    if(this.current === 1){
        if(this.anims[1].frame === this.anims[1].default){
            this.visible = false;
        } else {
            this.y -= 5;
        }
    }
};
var Spider = function(x,y){
    Actor.call(this, x, y);
    this.state = 0;
    this.range = 0;
    this.height = 38;
    let a = new Anim("images/spiderR10x40x38.png", 10, 40, true);
    this.anims.push(a);
    a = new Anim("images/spiderL10x40x38.png", 10, 40, true);
    this.anims.push(a);
    a = new Anim("images/spiderDie.png", 10, 40, false);
    a.default = 9;
    this.anims.push(a);
};
Spider.prototype = Object.create(Actor.prototype);
Spider.prototype.manageActions = function(){
    if(this.current === 2 && this.anims[2].frame === 
                             this.anims[2].default){
        this.visible = false; // dead and gone
    } else {
        this.collisionDetect();
        this.look();
        if(this.range){
        }
    }
};
Spider.prototype.collision = function(collisions){
    for(let i = 0; i < collisions.length; i++){
        switch(collisions[i].type){
            case "things":
            case "stops":
                if(this.state === 0){
                    this.resetCurrent((this.current === 0) ? 1 : 0);
                    this.moveX *= -1; // reverse 
                }
                break;
        }
    }
};
Spider.prototype.die = function(){
    if(this.state != flags.die){
        this.state = flags.die;
        this.moveX = 0;
        this.resetCurrent(2);
    }
};
Spider.prototype.look = function(){
    let cRect = character.getRect();
    let sRect = this.getRect();
    if(cRect.y + cRect.height === sRect.y + sRect.height){
        if(cRect.x < sRect.x) {
            if (this.moveX < 0){
                this.range = (cRect.x + cRect.width) - sRect.x;
            }
        }else if(this.moveX > 0) {
            this.range = cRect.x - (sRect.x + sRect.width);
        }
    } else {this.range = 0;}
};